#!/bin/sh
#
# lockdown:     various hacks to make exposed machines less vulnerable
#
# chkconfig:    345 99 0
# description:  remove module loading and direct memory access via /dev/mem
#               and provide more logging
#

# Source function library.
. /etc/rc.d/init.d/functions


DROP_RAWIO=0
DROP_MODULES=1
[ -r /etc/sysconfig/lockdown ] && . /etc/sysconfig/lockdown




get_bits()
{
    declare -a grep
    if [ -r /usr/include/linux/capability.h ]; then
	grep=( $( grep CAP_SYS_RAWIO /usr/include/linux/capability.h ) )
	rbit=${grep[2]}
	grep=( $( grep CAP_SYS_MODULE /usr/include/linux/capability.h ) )
	mbit=${grep[2]}
    else
	rbit=17
	mbit=16
	warning "Using default values for CAP_SYS_RAWIO and CAP_SYS_MODULE"
    fi
}

check_and_drop_caps()
{
    rbit=0
    mbit=0
    get_bits
    if [ "$DROP_RAWIO" -gt 0 ]; then
	if [ -z "$rbit" -o "$rbit" -lt "1" ]; then
	    echo -n "weird value $rbit for RAWIO cap, not doing anything"; failure; echo
	    return
	fi
    fi
    if [ "$DROP_MODULES" -gt 0 ]; then
	if [ -z "$mbit" -o "$mbit" -lt "1" ]; then
	    echo -n  "weird value $mbit for MODULE cap, not doing anything"; failure; echo
	    return
	fi
    fi

    read old < /proc/sys/kernel/cap-bound 2>/dev/null
    if [ -z "$old" ]; then
      echo -n "cannot get current bounding set value, already activated? "; passed; echo
      return
    fi
    if [ "$old" -gt "0" ]; then
	echo -n "weird values for current bounding set, not doing anything"; failure; echo
	return
    fi
    msg=""
    new=-1
    if [ "$DROP_RAWIO" -gt 0 ]; then
	new=$(( $new & (~( 1 << $rbit )) ))
        msg="$msg RAWIO"
    fi
    if [ "$DROP_MODULES" -gt 0 ]; then
        new=$(( $new & (~( 1 << $mbit )) ))
        msg="$msg MODULES"
    fi

    if [ "$old" -ne "$new" ]; then

#	printf "current cap set: 0x%x, need to change to 0x%x\n" "$old" "$new"

	echo "$new"  > /proc/sys/kernel/cap-bound

#	echo "wrote new restricted cap bounding set, re-reading"
        # no longer may be able to read - sign of success..
	read current < /proc/sys/kernel/cap-bound 2>/dev/null
	if [ "$current" -eq "$old" ]; then
	    echo -n "$0: cap bounding could NOT be changed" ; failure "cap_bound_change"; echo
	    return
	else
            old_txt=`printf  "0x%x" "$old"`
            new_txt=`printf  "0x%x" "$new"`
	    logger -t lockdown "cap bounding set: dropped $msg ($old_txt - $new_txt)"
	    echo -n "cap bounding set changed to $new_txt "; success "cap_bound_change: "; echo
	fi
    fi
}

lock_audit()
{
  if grep -q '4\.[0-9]' /etc/redhat-release; then
      passed "cannot lock audit subsystem on this OS version"
  else
      if [ ! -x /sbin/auditctl ]; then
	  failure "cannot find auditctl"
      else
	  echo -n "locking the audit subsystem"
      # this could presumably fail if the audit thing is already locked
      # but I don't see how to check "nicely" for this. We also cannot restart
      # auditd anymore.
	  /sbin/auditctl -e 2 && success "audit_lock: " || failure "audit_lock: "
	  echo
      fi
  fi
}

load_lockdown_module()
{
  # find module for current kernel
  LOCKDOWN=/lib/modules/`uname -r`/kernel/misc/lockdown/lockdown.ko
  # Start logging - this will take a few seconds.
  STRING="Starting lockdown logger: "
  echo -n $"$STRING"
  
  if [ -e /var/lock/subsys/lockdown ]; then
      echo -n "already active "
    warning "$STRING"
    return
  fi

  if [ ! -r "$LOCKDOWN" ]; then
      failure "cannot find lockdown module for current kernel `uname -r` "
  elif [ ! -x  /usr/bin/staprun ] ; then
      failure "cannot find staprun! "
  else 
      cd /
      ( /usr/bin/staprun "$LOCKDOWN" 2>&1 | logger -t lockdown ) </dev/null >&/dev/null &
      PID=$!
      kill -0 $PID 2>/dev/null
      RETVAL=$?
      if [ "$RETVAL" = 0 ]; then
	  touch /var/lock/subsys/lockdown
	  success "$STRING "
	    # don't bother with PIDs, the thing cannot be unloaded anyway
      else
	  failure "$STRING "
      fi
      echo
  fi
}

stop_hotplug()
{
    if [ -f /proc/sys/kernel/modprobe ]; then
      echo -n "turning off kernel hotplug"
      sysctl -w kernel.modprobe="/bin/true" >/dev/null 2>&1
      sysctl -w kernel.hotplug="/bin/true" >/dev/null 2>&1
      success "turn hotplug off: "
    fi
    pid=`/sbin/pidof -x udevd`
    if [ -n "$pid" ]; then
      # hm - looks like the thing can get restarted??
      kill $pid && success "turn udevd off: " || failure "turn udevd off: "
    fi
    
}
start() 
{
    stop_hotplug
    load_lockdown_module
    lock_audit
    check_and_drop_caps
}

# See how we were called.
case "$1" in
  start)
        start
        ;;
  stop)
        # not much we can do. Dropping capabilities is a one-way street..
        # but not sure whether chkconfig can handle "start"-only entries
	false || warning "cannot undo lockdown"
        ;;
  *)
        echo $"Usage: $0 start"
        exit 1
esac

exit 0
